#!/usr/bin/env python3

# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../../test/cpp/qps"))
subprocess.check_call(["./json_run_localhost_scenario_gen.py"])
subprocess.check_call(["./qps_json_driver_scenario_gen.py"])
subprocess.check_call(["buildifier", "-v", "-r", "."])

output = subprocess.check_output(["git", "status", "--porcelain"]).decode()
qps_json_driver_bzl = "test/cpp/qps/qps_json_driver_scenarios.bzl"
json_run_localhost_bzl = "test/cpp/qps/json_run_localhost_scenarios.bzl"

if qps_json_driver_bzl in output or json_run_localhost_bzl in output:
    print(
        "qps benchmark scenarios have been updated, please commit "
        "test/cpp/qps/qps_json_driver_scenarios.bzl and/or "
        "test/cpp/qps/json_run_localhost_scenarios.bzl"
    )
    sys.exit(1)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""AsyncIO version of Channelz servicer."""

from grpc.experimental import aio
from grpc_channelz.v1._servicer import ChannelzServicer as _SyncChannelzServicer
import grpc_channelz.v1.channelz_pb2 as _channelz_pb2
import grpc_channelz.v1.channelz_pb2_grpc as _channelz_pb2_grpc


class ChannelzServicer(_channelz_pb2_grpc.ChannelzServicer):
    """AsyncIO servicer for handling RPCs for service statuses."""

    @staticmethod
    async def GetTopChannels(
        request: _channelz_pb2.GetTopChannelsRequest,
        context: aio.ServicerContext,
    ) -> _channelz_pb2.GetTopChannelsResponse:
        return _SyncChannelzServicer.GetTopChannels(request, context)

    @staticmethod
    async def GetServers(
        request: _channelz_pb2.GetServersRequest, context: aio.ServicerContext
    ) -> _channelz_pb2.GetServersResponse:
        return _SyncChannelzServicer.GetServers(request, context)

    @staticmethod
    async def GetServer(
        request: _channelz_pb2.GetServerRequest, context: aio.ServicerContext
    ) -> _channelz_pb2.GetServerResponse:
        return _SyncChannelzServicer.GetServer(request, context)

    @staticmethod
    async def GetServerSockets(
        request: _channelz_pb2.GetServerSocketsRequest,
        context: aio.ServicerContext,
    ) -> _channelz_pb2.GetServerSocketsResponse:
        return _SyncChannelzServicer.GetServerSockets(request, context)

    @staticmethod
    async def GetChannel(
        request: _channelz_pb2.GetChannelRequest, context: aio.ServicerContext
    ) -> _channelz_pb2.GetChannelResponse:
        return _SyncChannelzServicer.GetChannel(request, context)

    @staticmethod
    async def GetSubchannel(
        request: _channelz_pb2.GetSubchannelRequest,
        context: aio.ServicerContext,
    ) -> _channelz_pb2.GetSubchannelResponse:
        return _SyncChannelzServicer.GetSubchannel(request, context)

    @staticmethod
    async def GetSocket(
        request: _channelz_pb2.GetSocketRequest, context: aio.ServicerContext
    ) -> _channelz_pb2.GetSocketResponse:
        return _SyncChannelzServicer.GetSocket(request, context)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
from contextvars import ContextVar
import datetime
from typing import Optional

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit import resources
from tests_aio.unit import _constants

_INITIAL_METADATA_KEY = "x-grpc-test-echo-initial"
_TRAILING_METADATA_KEY = "x-grpc-test-echo-trailing-bin"

TEST_CONTEXT_VAR: ContextVar[str] = ContextVar("")


async def _maybe_echo_metadata(servicer_context):
    """Copies metadata from request to response if it is present."""
    invocation_metadata = dict(servicer_context.invocation_metadata())
    if _INITIAL_METADATA_KEY in invocation_metadata:
        initial_metadatum = (
            _INITIAL_METADATA_KEY,
            invocation_metadata[_INITIAL_METADATA_KEY],
        )
        await servicer_context.send_initial_metadata((initial_metadatum,))
    if _TRAILING_METADATA_KEY in invocation_metadata:
        trailing_metadatum = (
            _TRAILING_METADATA_KEY,
            invocation_metadata[_TRAILING_METADATA_KEY],
        )
        servicer_context.set_trailing_metadata((trailing_metadatum,))


async def _maybe_echo_status(
    request: messages_pb2.SimpleRequest, servicer_context
):
    """Echos the RPC status if demanded by the request."""
    if request.HasField("response_status"):
        await servicer_context.abort(
            request.response_status.code, request.response_status.message
        )


class TestServiceServicer(test_pb2_grpc.TestServiceServicer):
    def __init__(self, record: Optional[list] = None):
        self.record = record if record is not None else []

    def _append_to_log(self):
        self.record.append("servicer:" + TEST_CONTEXT_VAR.get("service"))

    async def UnaryCall(self, request, context):
        self._append_to_log()
        await _maybe_echo_metadata(context)
        await _maybe_echo_status(request, context)
        return messages_pb2.SimpleResponse(
            payload=messages_pb2.Payload(
                type=messages_pb2.COMPRESSABLE,
                body=b"\x00" * request.response_size,
            )
        )

    async def EmptyCall(self, request, context):
        self._append_to_log()
        return empty_pb2.Empty()

    async def StreamingOutputCall(
        self, request: messages_pb2.StreamingOutputCallRequest, unused_context
    ):
        self._append_to_log()
        for response_parameters in request.response_parameters:
            if response_parameters.interval_us != 0:
                await asyncio.sleep(
                    datetime.timedelta(
                        microseconds=response_parameters.interval_us
                    ).total_seconds()
                )
            if response_parameters.size != 0:
                yield messages_pb2.StreamingOutputCallResponse(
                    payload=messages_pb2.Payload(
                        type=request.response_type,
                        body=b"\x00" * response_parameters.size,
                    )
                )
            else:
                yield messages_pb2.StreamingOutputCallResponse()

    # Next methods are extra ones that are registered programmatically
    # when the sever is instantiated. They are not being provided by
    # the proto file.
    async def UnaryCallWithSleep(self, unused_request, unused_context):
        self._append_to_log()
        await asyncio.sleep(_constants.UNARY_CALL_WITH_SLEEP_VALUE)
        return messages_pb2.SimpleResponse()

    async def StreamingInputCall(self, request_async_iterator, unused_context):
        self._append_to_log()
        aggregate_size = 0
        async for request in request_async_iterator:
            if request.payload is not None and request.payload.body:
                aggregate_size += len(request.payload.body)
        return messages_pb2.StreamingInputCallResponse(
            aggregated_payload_size=aggregate_size
        )

    async def FullDuplexCall(self, request_async_iterator, context):
        self._append_to_log()
        await _maybe_echo_metadata(context)
        async for request in request_async_iterator:
            await _maybe_echo_status(request, context)
            for response_parameters in request.response_parameters:
                if response_parameters.interval_us != 0:
                    await asyncio.sleep(
                        datetime.timedelta(
                            microseconds=response_parameters.interval_us
                        ).total_seconds()
                    )
                if response_parameters.size != 0:
                    yield messages_pb2.StreamingOutputCallResponse(
                        payload=messages_pb2.Payload(
                            type=request.payload.type,
                            body=b"\x00" * response_parameters.size,
                        )
                    )
                else:
                    yield messages_pb2.StreamingOutputCallResponse()


def _create_extra_generic_handler(servicer: TestServiceServicer):
    # Add programmatically extra methods not provided by the proto file
    # that are used during the tests
    rpc_method_handlers = {
        "UnaryCallWithSleep": grpc.unary_unary_rpc_method_handler(
            servicer.UnaryCallWithSleep,
            request_deserializer=messages_pb2.SimpleRequest.FromString,
            response_serializer=messages_pb2.SimpleResponse.SerializeToString,
        )
    }
    return grpc.method_handlers_generic_handler(
        "grpc.testing.TestService", rpc_method_handlers
    )


async def start_test_server(
    port=0,
    secure=False,
    server_credentials=None,
    interceptors=None,
    record: Optional[list] = None,
):
    server = aio.server(
        options=(("grpc.so_reuseport", 0),), interceptors=interceptors
    )
    servicer = TestServiceServicer(record)
    test_pb2_grpc.add_TestServiceServicer_to_server(servicer, server)

    server.add_generic_rpc_handlers((_create_extra_generic_handler(servicer),))

    if secure:
        if server_credentials is None:
            server_credentials = grpc.ssl_server_credentials(
                [(resources.private_key(), resources.certificate_chain())]
            )
        port = server.add_secure_port("[::]:%d" % port, server_credentials)
    else:
        port = server.add_insecure_port("[::]:%d" % port)

    await server.start()

    # NOTE(lidizheng) returning the server to prevent it from deallocation
    return "localhost:%d" % port, server



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCenvoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto\x12\x35\x65nvoy.extensions.router.cluster_specifiers.matcher.v3\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\")\n\rClusterAction\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"Z\n\x17MatcherClusterSpecifier\x12?\n\x0f\x63luster_matcher\x18\x01 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xc5\x01\nCio.envoyproxy.envoy.extensions.router.cluster_specifiers.matcher.v3B\x0cMatcherProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/router/cluster_specifiers/matcher/v3;matcherv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.router.cluster_specifiers.matcher.v3.matcher_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nCio.envoyproxy.envoy.extensions.router.cluster_specifiers.matcher.v3B\014MatcherProtoP\001Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/router/cluster_specifiers/matcher/v3;matcherv3\272\200\310\321\006\002\020\002'
  _globals['_CLUSTERACTION'].fields_by_name['cluster']._loaded_options = None
  _globals['_CLUSTERACTION'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_MATCHERCLUSTERSPECIFIER'].fields_by_name['cluster_matcher']._loaded_options = None
  _globals['_MATCHERCLUSTERSPECIFIER'].fields_by_name['cluster_matcher']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CLUSTERACTION']._serialized_start=217
  _globals['_CLUSTERACTION']._serialized_end=258
  _globals['_MATCHERCLUSTERSPECIFIER']._serialized_start=260
  _globals['_MATCHERCLUSTERSPECIFIER']._serialized_end=350
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/http/v3/path_transformation.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/http/v3/path_transformation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,envoy/type/http/v3/path_transformation.proto\x12\x12\x65nvoy.type.http.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xee\x02\n\x12PathTransformation\x12\x44\n\noperations\x18\x01 \x03(\x0b\x32\x30.envoy.type.http.v3.PathTransformation.Operation\x1a\x91\x02\n\tOperation\x12h\n\x17normalize_path_rfc_3986\x18\x02 \x01(\x0b\x32\x45.envoy.type.http.v3.PathTransformation.Operation.NormalizePathRFC3986H\x00\x12V\n\rmerge_slashes\x18\x03 \x01(\x0b\x32=.envoy.type.http.v3.PathTransformation.Operation.MergeSlashesH\x00\x1a\x16\n\x14NormalizePathRFC3986\x1a\x0e\n\x0cMergeSlashesB\x1a\n\x13operation_specifier\x12\x03\xf8\x42\x01\x42\x87\x01\n io.envoyproxy.envoy.type.http.v3B\x17PathTransformationProtoP\x01Z@github.com/envoyproxy/go-control-plane/envoy/type/http/v3;httpv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.http.v3.path_transformation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n io.envoyproxy.envoy.type.http.v3B\027PathTransformationProtoP\001Z@github.com/envoyproxy/go-control-plane/envoy/type/http/v3;httpv3\272\200\310\321\006\002\020\002'
  _globals['_PATHTRANSFORMATION_OPERATION'].oneofs_by_name['operation_specifier']._loaded_options = None
  _globals['_PATHTRANSFORMATION_OPERATION'].oneofs_by_name['operation_specifier']._serialized_options = b'\370B\001'
  _globals['_PATHTRANSFORMATION']._serialized_start=125
  _globals['_PATHTRANSFORMATION']._serialized_end=491
  _globals['_PATHTRANSFORMATION_OPERATION']._serialized_start=218
  _globals['_PATHTRANSFORMATION_OPERATION']._serialized_end=491
  _globals['_PATHTRANSFORMATION_OPERATION_NORMALIZEPATHRFC3986']._serialized_start=425
  _globals['_PATHTRANSFORMATION_OPERATION_NORMALIZEPATHRFC3986']._serialized_end=447
  _globals['_PATHTRANSFORMATION_OPERATION_MERGESLASHES']._serialized_start=449
  _globals['_PATHTRANSFORMATION_OPERATION_MERGESLASHES']._serialized_end=463
# @@protoc_insertion_point(module_scope)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests metadata flags feature by testing wait-for-ready semantics"""

import logging
import queue
import socket
import threading
import time
import unittest
import weakref

import grpc

from tests.unit import test_common
import tests.unit.framework.common
from tests.unit.framework.common import get_socket
from tests.unit.framework.common import test_constants

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"


def handle_unary_unary(test, request, servicer_context):
    return _RESPONSE


def handle_unary_stream(test, request, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(test, request_iterator, servicer_context):
    for _ in request_iterator:
        pass
    return _RESPONSE


def handle_stream_stream(test, request_iterator, servicer_context):
    for _ in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, test, request_streaming, response_streaming):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = lambda req, ctx: handle_stream_stream(
                test, req, ctx
            )
        elif self.request_streaming:
            self.stream_unary = lambda req, ctx: handle_stream_unary(
                test, req, ctx
            )
        elif self.response_streaming:
            self.unary_stream = lambda req, ctx: handle_unary_stream(
                test, req, ctx
            )
        else:
            self.unary_unary = lambda req, ctx: handle_unary_unary(
                test, req, ctx
            )


def get_method_handlers(test):
    return {
        _UNARY_UNARY: _MethodHandler(test, False, False),
        _UNARY_STREAM: _MethodHandler(test, False, True),
        _STREAM_UNARY: _MethodHandler(test, True, False),
        _STREAM_STREAM: _MethodHandler(test, True, True),
    }


def create_phony_channel():
    """Creating phony channels is a workaround for retries"""
    host, port, sock = get_socket(sock_options=(socket.SO_REUSEADDR,))
    sock.close()
    return grpc.insecure_channel("{}:{}".format(host, port))


def perform_unary_unary_call(channel, wait_for_ready=None):
    channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    ).__call__(
        _REQUEST,
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


def perform_unary_unary_with_call(channel, wait_for_ready=None):
    channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    ).with_call(
        _REQUEST,
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


def perform_unary_unary_future(channel, wait_for_ready=None):
    channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    ).future(
        _REQUEST,
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    ).result(
        timeout=test_constants.LONG_TIMEOUT
    )


def perform_unary_stream_call(channel, wait_for_ready=None):
    response_iterator = channel.unary_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
        _registered_method=True,
    ).__call__(
        _REQUEST,
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )
    for _ in response_iterator:
        pass


def perform_stream_unary_call(channel, wait_for_ready=None):
    channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        _registered_method=True,
    ).__call__(
        iter([_REQUEST] * test_constants.STREAM_LENGTH),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


def perform_stream_unary_with_call(channel, wait_for_ready=None):
    channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        _registered_method=True,
    ).with_call(
        iter([_REQUEST] * test_constants.STREAM_LENGTH),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


def perform_stream_unary_future(channel, wait_for_ready=None):
    channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        _registered_method=True,
    ).future(
        iter([_REQUEST] * test_constants.STREAM_LENGTH),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    ).result(
        timeout=test_constants.LONG_TIMEOUT
    )


def perform_stream_stream_call(channel, wait_for_ready=None):
    response_iterator = channel.stream_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
        _registered_method=True,
    ).__call__(
        iter([_REQUEST] * test_constants.STREAM_LENGTH),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )
    for _ in response_iterator:
        pass


_ALL_CALL_CASES = [
    perform_unary_unary_call,
    perform_unary_unary_with_call,
    perform_unary_unary_future,
    perform_unary_stream_call,
    perform_stream_unary_call,
    perform_stream_unary_with_call,
    perform_stream_unary_future,
    perform_stream_stream_call,
]


class MetadataFlagsTest(unittest.TestCase):
    def check_connection_does_failfast(self, fn, channel, wait_for_ready=None):
        try:
            fn(channel, wait_for_ready)
            self.fail("The Call should fail")
        except BaseException as e:  # pylint: disable=broad-except
            self.assertIs(grpc.StatusCode.UNAVAILABLE, e.code())

    def test_call_wait_for_ready_default(self):
        for perform_call in _ALL_CALL_CASES:
            with create_phony_channel() as channel:
                self.check_connection_does_failfast(perform_call, channel)

    def test_call_wait_for_ready_disabled(self):
        for perform_call in _ALL_CALL_CASES:
            with create_phony_channel() as channel:
                self.check_connection_does_failfast(
                    perform_call, channel, wait_for_ready=False
                )

    def test_call_wait_for_ready_enabled(self):
        # To test the wait mechanism, Python thread is required to make
        #   client set up first without handling them case by case.
        # Also, Python thread don't pass the unhandled exceptions to
        #   main thread. So, it needs another method to store the
        #   exceptions and raise them again in main thread.
        unhandled_exceptions = queue.Queue()

        # We just need an unused TCP port
        host, port, sock = get_socket(sock_options=(socket.SO_REUSEADDR,))
        sock.close()

        addr = "{}:{}".format(host, port)
        wg = test_common.WaitGroup(len(_ALL_CALL_CASES))

        def wait_for_transient_failure(channel_connectivity):
            if (
                channel_connectivity
                == grpc.ChannelConnectivity.TRANSIENT_FAILURE
            ):
                wg.done()

        def test_call(perform_call):
            with grpc.insecure_channel(addr) as channel:
                try:
                    channel.subscribe(wait_for_transient_failure)
                    perform_call(channel, wait_for_ready=True)
                except BaseException as e:  # pylint: disable=broad-except
                    # If the call failed, the thread would be destroyed. The
                    # channel object can be collected before calling the
                    # callback, which will result in a deadlock.
                    wg.done()
                    unhandled_exceptions.put(e, True)

        test_threads = []
        for perform_call in _ALL_CALL_CASES:
            test_thread = threading.Thread(
                target=test_call, args=(perform_call,)
            )
            test_thread.daemon = True
            test_thread.exception = None
            test_thread.start()
            test_threads.append(test_thread)

        # Start the server after the connections are waiting
        wg.wait()
        server = test_common.test_server(reuse_port=True)
        server.add_registered_method_handlers(
            _SERVICE_NAME, get_method_handlers(weakref.proxy(self))
        )
        server.add_insecure_port(addr)
        server.start()

        for test_thread in test_threads:
            test_thread.join()

        # Stop the server to make test end properly
        server.stop(0)

        if not unhandled_exceptions.empty():
            raise unhandled_exceptions.get(True)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for grpc.framework.foundation.logging_pool."""

import threading
import unittest

from grpc.framework.foundation import logging_pool

_POOL_SIZE = 16


class _CallableObject:
    def __init__(self):
        self._lock = threading.Lock()
        self._passed_values = []

    def __call__(self, value):
        with self._lock:
            self._passed_values.append(value)

    def passed_values(self):
        with self._lock:
            return tuple(self._passed_values)


class LoggingPoolTest(unittest.TestCase):
    def testUpAndDown(self):
        pool = logging_pool.pool(_POOL_SIZE)
        pool.shutdown(wait=True)

        with logging_pool.pool(_POOL_SIZE) as pool:
            self.assertIsNotNone(pool)

    def testTaskExecuted(self):
        test_list = []

        with logging_pool.pool(_POOL_SIZE) as pool:
            pool.submit(lambda: test_list.append(object())).result()

        self.assertTrue(test_list)

    def testException(self):
        with logging_pool.pool(_POOL_SIZE) as pool:
            raised_exception = pool.submit(lambda: 1 / 0).exception()

        self.assertIsNotNone(raised_exception)

    def testCallableObjectExecuted(self):
        callable_object = _CallableObject()
        passed_object = object()
        with logging_pool.pool(_POOL_SIZE) as pool:
            future = pool.submit(callable_object, passed_object)
        self.assertIsNone(future.result())
        self.assertSequenceEqual(
            (passed_object,), callable_object.passed_values()
        )


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2020 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_aio.unit._common import CountingRequestIterator
from tests_aio.unit._common import CountingResponseIterator
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_NUM_STREAM_RESPONSES = 5
_NUM_STREAM_REQUESTS = 5
_RESPONSE_PAYLOAD_SIZE = 7


class _StreamStreamInterceptorEmpty(aio.StreamStreamClientInterceptor):
    async def intercept_stream_stream(
        self, continuation, client_call_details, request_iterator
    ):
        return await continuation(client_call_details, request_iterator)

    def assert_in_final_state(self, test: unittest.TestCase):
        pass


class _StreamStreamInterceptorWithRequestAndResponseIterator(
    aio.StreamStreamClientInterceptor
):
    async def intercept_stream_stream(
        self, continuation, client_call_details, request_iterator
    ):
        self.request_iterator = CountingRequestIterator(request_iterator)
        call = await continuation(client_call_details, self.request_iterator)
        self.response_iterator = CountingResponseIterator(call)
        return self.response_iterator

    def assert_in_final_state(self, test: unittest.TestCase):
        test.assertEqual(
            _NUM_STREAM_REQUESTS, self.request_iterator.request_cnt
        )
        test.assertEqual(
            _NUM_STREAM_RESPONSES, self.response_iterator.response_cnt
        )


class TestStreamStreamClientInterceptor(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    async def test_intercepts(self):
        for interceptor_class in (
            _StreamStreamInterceptorEmpty,
            _StreamStreamInterceptorWithRequestAndResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()
                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                # Prepares the request
                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.append(
                    messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
                )

                async def request_iterator():
                    for _ in range(_NUM_STREAM_REQUESTS):
                        yield request

                call = stub.FullDuplexCall(request_iterator())

                await call.wait_for_connection()

                response_cnt = 0
                async for response in call:
                    response_cnt += 1
                    self.assertIsInstance(
                        response, messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                self.assertEqual(response_cnt, _NUM_STREAM_RESPONSES)
                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_intercepts_using_write_and_read(self):
        for interceptor_class in (
            _StreamStreamInterceptorEmpty,
            _StreamStreamInterceptorWithRequestAndResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptor = interceptor_class()
                channel = aio.insecure_channel(
                    self._server_target, interceptors=[interceptor]
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                # Prepares the request
                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.append(
                    messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
                )

                call = stub.FullDuplexCall()

                for _ in range(_NUM_STREAM_RESPONSES):
                    await call.write(request)
                    response = await call.read()
                    self.assertIsInstance(
                        response, messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                await call.done_writing()

                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                interceptor.assert_in_final_state(self)

                await channel.close()

    async def test_multiple_interceptors_request_iterator(self):
        for interceptor_class in (
            _StreamStreamInterceptorEmpty,
            _StreamStreamInterceptorWithRequestAndResponseIterator,
        ):
            with self.subTest(name=interceptor_class):
                interceptors = [interceptor_class(), interceptor_class()]
                channel = aio.insecure_channel(
                    self._server_target, interceptors=interceptors
                )
                stub = test_pb2_grpc.TestServiceStub(channel)

                # Prepares the request
                request = messages_pb2.StreamingOutputCallRequest()
                request.response_parameters.append(
                    messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
                )

                call = stub.FullDuplexCall()

                for _ in range(_NUM_STREAM_RESPONSES):
                    await call.write(request)
                    response = await call.read()
                    self.assertIsInstance(
                        response, messages_pb2.StreamingOutputCallResponse
                    )
                    self.assertEqual(
                        _RESPONSE_PAYLOAD_SIZE, len(response.payload.body)
                    )

                await call.done_writing()

                self.assertEqual(await call.code(), grpc.StatusCode.OK)
                self.assertEqual(await call.initial_metadata(), aio.Metadata())
                self.assertEqual(await call.trailing_metadata(), aio.Metadata())
                self.assertEqual(await call.details(), "")
                self.assertEqual(await call.debug_error_string(), "")
                self.assertEqual(call.cancel(), False)
                self.assertEqual(call.cancelled(), False)
                self.assertEqual(call.done(), True)

                for interceptor in interceptors:
                    interceptor.assert_in_final_state(self)

                await channel.close()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/transform/v3/transform.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/transform/v3/transform.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.common.mutation_rules.v3 import mutation_rules_pb2 as envoy_dot_config_dot_common_dot_mutation__rules_dot_v3_dot_mutation__rules__pb2
from envoy.config.core.v3 import substitution_format_string_pb2 as envoy_dot_config_dot_core_dot_v3_dot_substitution__format__string__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:envoy/extensions/filters/http/transform/v3/transform.proto\x12*envoy.extensions.filters.http.transform.v3\x1a:envoy/config/common/mutation_rules/v3/mutation_rules.proto\x1a\x35\x65nvoy/config/core/v3/substitution_format_string.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x02\n\x0fTransformConfig\x12Z\n\x16request_transformation\x18\x01 \x01(\x0b\x32:.envoy.extensions.filters.http.transform.v3.Transformation\x12[\n\x17response_transformation\x18\x02 \x01(\x0b\x32:.envoy.extensions.filters.http.transform.v3.Transformation\x12\x1b\n\x13\x63lear_cluster_cache\x18\x03 \x01(\x08\x12\x19\n\x11\x63lear_route_cache\x18\x04 \x01(\x08\"\xbf\x01\n\x0eTransformation\x12P\n\x11headers_mutations\x18\x01 \x03(\x0b\x32\x35.envoy.config.common.mutation_rules.v3.HeaderMutation\x12[\n\x13\x62ody_transformation\x18\x02 \x01(\x0b\x32>.envoy.extensions.filters.http.transform.v3.BodyTransformation\"\xee\x01\n\x12\x42odyTransformation\x12M\n\x0b\x62ody_format\x18\x01 \x01(\x0b\x32..envoy.config.core.v3.SubstitutionFormatStringB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12^\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32N.envoy.extensions.filters.http.transform.v3.BodyTransformation.TransformAction\")\n\x0fTransformAction\x12\t\n\x05MERGE\x10\x00\x12\x0b\n\x07REPLACE\x10\x01\x42\xb3\x01\n8io.envoyproxy.envoy.extensions.filters.http.transform.v3B\x0eTransformProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/transform/v3;transformv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.transform.v3.transform_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n8io.envoyproxy.envoy.extensions.filters.http.transform.v3B\016TransformProtoP\001Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/transform/v3;transformv3\272\200\310\321\006\002\020\002'
  _globals['_BODYTRANSFORMATION'].fields_by_name['body_format']._loaded_options = None
  _globals['_BODYTRANSFORMATION'].fields_by_name['body_format']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TRANSFORMCONFIG']._serialized_start=278
  _globals['_TRANSFORMCONFIG']._serialized_end=536
  _globals['_TRANSFORMATION']._serialized_start=539
  _globals['_TRANSFORMATION']._serialized_end=730
  _globals['_BODYTRANSFORMATION']._serialized_start=733
  _globals['_BODYTRANSFORMATION']._serialized_end=971
  _globals['_BODYTRANSFORMATION_TRANSFORMACTION']._serialized_start=930
  _globals['_BODYTRANSFORMATION_TRANSFORMACTION']._serialized_end=971
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC interoperability test client."""

import os

from absl import app
from absl.flags import argparse_flags
from google import auth as google_auth
from google.auth import jwt as google_auth_jwt
import grpc

from src.proto.grpc.testing import test_pb2_grpc
from tests.interop import methods
from tests.interop import resources


def parse_interop_client_args(argv):
    parser = argparse_flags.ArgumentParser()
    parser.add_argument(
        "--server_host",
        default="localhost",
        type=str,
        help="the host to which to connect",
    )
    parser.add_argument(
        "--server_port",
        type=int,
        required=True,
        help="the port to which to connect",
    )
    parser.add_argument(
        "--test_case",
        default="large_unary",
        type=str,
        help="the test case to execute",
    )
    parser.add_argument(
        "--use_tls",
        default=False,
        type=resources.parse_bool,
        help="require a secure connection",
    )
    parser.add_argument(
        "--use_alts",
        default=False,
        type=resources.parse_bool,
        help="require an ALTS secure connection",
    )
    parser.add_argument(
        "--use_test_ca",
        default=False,
        type=resources.parse_bool,
        help="replace platform root CAs with ca.pem",
    )
    parser.add_argument(
        "--custom_credentials_type",
        choices=["compute_engine_channel_creds"],
        default=None,
        help="use google default credentials",
    )
    parser.add_argument(
        "--server_host_override",
        type=str,
        help="the server host to which to claim to connect",
    )
    parser.add_argument(
        "--oauth_scope", type=str, help="scope for OAuth tokens"
    )
    parser.add_argument(
        "--default_service_account",
        type=str,
        help="email address of the default service account",
    )
    parser.add_argument(
        "--grpc_test_use_grpclb_with_child_policy",
        type=str,
        help=(
            "If non-empty, set a static service config on channels created by "
            + "grpc::CreateTestChannel, that configures the grpclb LB policy "
            + "with a child policy being the value of this flag (e.g."
            " round_robin " + "or pick_first)."
        ),
    )
    return parser.parse_args(argv[1:])


def _create_call_credentials(args):
    if args.test_case == "oauth2_auth_token":
        google_credentials, unused_project_id = google_auth.default(
            scopes=[args.oauth_scope]
        )
        google_credentials.refresh(google_auth.transport.requests.Request())
        return grpc.access_token_call_credentials(google_credentials.token)
    elif args.test_case == "compute_engine_creds":
        google_credentials, unused_project_id = google_auth.default(
            scopes=[args.oauth_scope]
        )
        return grpc.metadata_call_credentials(
            google_auth.transport.grpc.AuthMetadataPlugin(
                credentials=google_credentials,
                request=google_auth.transport.requests.Request(),
            )
        )
    elif args.test_case == "jwt_token_creds":
        google_credentials = (
            google_auth_jwt.OnDemandCredentials.from_service_account_file(
                os.environ[google_auth.environment_vars.CREDENTIALS]
            )
        )
        return grpc.metadata_call_credentials(
            google_auth.transport.grpc.AuthMetadataPlugin(
                credentials=google_credentials, request=None
            )
        )
    else:
        return None


def get_secure_channel_parameters(args):
    call_credentials = _create_call_credentials(args)

    channel_opts = ()
    if args.grpc_test_use_grpclb_with_child_policy:
        channel_opts += (
            (
                "grpc.service_config",
                '{"loadBalancingConfig": [{"grpclb": {"childPolicy": [{"%s":'
                " {}}]}}]}" % args.grpc_test_use_grpclb_with_child_policy,
            ),
        )
    if args.custom_credentials_type is not None:
        if args.custom_credentials_type == "compute_engine_channel_creds":
            assert call_credentials is None
            google_credentials, unused_project_id = google_auth.default(
                scopes=[args.oauth_scope]
            )
            call_creds = grpc.metadata_call_credentials(
                google_auth.transport.grpc.AuthMetadataPlugin(
                    credentials=google_credentials,
                    request=google_auth.transport.requests.Request(),
                )
            )
            channel_credentials = grpc.compute_engine_channel_credentials(
                call_creds
            )
        else:
            raise ValueError(
                "Unknown credentials type '{}'".format(
                    args.custom_credentials_type
                )
            )
    elif args.use_tls:
        if args.use_test_ca:
            root_certificates = resources.test_root_certificates()
        else:
            root_certificates = None  # will load default roots.

        channel_credentials = grpc.ssl_channel_credentials(root_certificates)
        if call_credentials is not None:
            channel_credentials = grpc.composite_channel_credentials(
                channel_credentials, call_credentials
            )

        if args.server_host_override:
            channel_opts += (
                (
                    "grpc.ssl_target_name_override",
                    args.server_host_override,
                ),
            )
    elif args.use_alts:
        channel_credentials = grpc.alts_channel_credentials()

    return channel_credentials, channel_opts


def _create_channel(args):
    target = "{}:{}".format(args.server_host, args.server_port)

    if (
        args.use_tls
        or args.use_alts
        or args.custom_credentials_type is not None
    ):
        channel_credentials, options = get_secure_channel_parameters(args)
        return grpc.secure_channel(target, channel_credentials, options)
    else:
        return grpc.insecure_channel(target)


def create_stub(channel, args):
    if args.test_case == "unimplemented_service":
        return test_pb2_grpc.UnimplementedServiceStub(channel)
    else:
        return test_pb2_grpc.TestServiceStub(channel)


def _test_case_from_arg(test_case_arg):
    for test_case in methods.TestCase:
        if test_case_arg == test_case.value:
            return test_case
    else:
        raise ValueError('No test case "%s"!' % test_case_arg)


def test_interoperability(args):
    channel = _create_channel(args)
    stub = create_stub(channel, args)
    test_case = _test_case_from_arg(args.test_case)
    test_case.test_interoperability(stub, args)


if __name__ == "__main__":
    app.run(test_interoperability, flags_parser=parse_interop_client_args)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/service/status/v2/csds.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/service/status/v2/csds.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.admin.v2alpha import config_dump_pb2 as envoy_dot_admin_dot_v2alpha_dot_config__dump__pb2
from envoy.api.v2.core import base_pb2 as envoy_dot_api_dot_v2_dot_core_dot_base__pb2
try:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2
except AttributeError:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy.api.v2.core.socket_option_pb2
from envoy.type.matcher import node_pb2 as envoy_dot_type_dot_matcher_dot_node__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"envoy/service/status/v2/csds.proto\x12\x17\x65nvoy.service.status.v2\x1a%envoy/admin/v2alpha/config_dump.proto\x1a\x1c\x65nvoy/api/v2/core/base.proto\x1a\x1d\x65nvoy/type/matcher/node.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1dudpa/annotations/status.proto\"M\n\x13\x43lientStatusRequest\x12\x36\n\rnode_matchers\x18\x01 \x03(\x0b\x32\x1f.envoy.type.matcher.NodeMatcher\"\xea\x02\n\x0cPerXdsConfig\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.envoy.service.status.v2.ConfigStatus\x12\x43\n\x0flistener_config\x18\x02 \x01(\x0b\x32(.envoy.admin.v2alpha.ListenersConfigDumpH\x00\x12\x41\n\x0e\x63luster_config\x18\x03 \x01(\x0b\x32\'.envoy.admin.v2alpha.ClustersConfigDumpH\x00\x12=\n\x0croute_config\x18\x04 \x01(\x0b\x32%.envoy.admin.v2alpha.RoutesConfigDumpH\x00\x12J\n\x13scoped_route_config\x18\x05 \x01(\x0b\x32+.envoy.admin.v2alpha.ScopedRoutesConfigDumpH\x00\x42\x10\n\x0eper_xds_config\"p\n\x0c\x43lientConfig\x12%\n\x04node\x18\x01 \x01(\x0b\x32\x17.envoy.api.v2.core.Node\x12\x39\n\nxds_config\x18\x02 \x03(\x0b\x32%.envoy.service.status.v2.PerXdsConfig\"M\n\x14\x43lientStatusResponse\x12\x35\n\x06\x63onfig\x18\x01 \x03(\x0b\x32%.envoy.service.status.v2.ClientConfig*K\n\x0c\x43onfigStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06SYNCED\x10\x01\x12\x0c\n\x08NOT_SENT\x10\x02\x12\t\n\x05STALE\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x32\xb2\x02\n\x1c\x43lientStatusDiscoveryService\x12w\n\x12StreamClientStatus\x12,.envoy.service.status.v2.ClientStatusRequest\x1a-.envoy.service.status.v2.ClientStatusResponse\"\x00(\x01\x30\x01\x12\x98\x01\n\x11\x46\x65tchClientStatus\x12,.envoy.service.status.v2.ClientStatusRequest\x1a-.envoy.service.status.v2.ClientStatusResponse\"&\x82\xd3\xe4\x93\x02 \"\x1b/v2/discovery:client_status:\x01*B\x85\x01\n%io.envoyproxy.envoy.service.status.v2B\tCsdsProtoP\x01ZGgithub.com/envoyproxy/go-control-plane/envoy/service/status/v2;statusv2\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.service.status.v2.csds_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n%io.envoyproxy.envoy.service.status.v2B\tCsdsProtoP\001ZGgithub.com/envoyproxy/go-control-plane/envoy/service/status/v2;statusv2\272\200\310\321\006\002\020\001'
  _globals['_CLIENTSTATUSDISCOVERYSERVICE'].methods_by_name['FetchClientStatus']._loaded_options = None
  _globals['_CLIENTSTATUSDISCOVERYSERVICE'].methods_by_name['FetchClientStatus']._serialized_options = b'\202\323\344\223\002 \"\033/v2/discovery:client_status:\001*'
  _globals['_CONFIGSTATUS']._serialized_start=861
  _globals['_CONFIGSTATUS']._serialized_end=936
  _globals['_CLIENTSTATUSREQUEST']._serialized_start=224
  _globals['_CLIENTSTATUSREQUEST']._serialized_end=301
  _globals['_PERXDSCONFIG']._serialized_start=304
  _globals['_PERXDSCONFIG']._serialized_end=666
  _globals['_CLIENTCONFIG']._serialized_start=668
  _globals['_CLIENTCONFIG']._serialized_end=780
  _globals['_CLIENTSTATUSRESPONSE']._serialized_start=782
  _globals['_CLIENTSTATUSRESPONSE']._serialized_end=859
  _globals['_CLIENTSTATUSDISCOVERYSERVICE']._serialized_start=939
  _globals['_CLIENTSTATUSDISCOVERYSERVICE']._serialized_end=1245
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/api/v2/auth/tls.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/api/v2/auth/tls.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.auth import common_pb2 as envoy_dot_api_dot_v2_dot_auth_dot_common__pb2
from envoy.api.v2.auth import secret_pb2 as envoy_dot_api_dot_v2_dot_auth_dot_secret__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x65nvoy/api/v2/auth/tls.proto\x12\x11\x65nvoy.api.v2.auth\x1a\x1e\x65nvoy/api/v2/auth/common.proto\x1a\x1e\x65nvoy/api/v2/auth/secret.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xc1\x01\n\x12UpstreamTlsContext\x12?\n\x12\x63ommon_tls_context\x18\x01 \x01(\x0b\x32#.envoy.api.v2.auth.CommonTlsContext\x12\x15\n\x03sni\x18\x02 \x01(\tB\x08\xfa\x42\x05r\x03(\xff\x01\x12\x1b\n\x13\x61llow_renegotiation\x18\x03 \x01(\x08\x12\x36\n\x10max_session_keys\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\"\xf7\x03\n\x14\x44ownstreamTlsContext\x12?\n\x12\x63ommon_tls_context\x18\x01 \x01(\x0b\x32#.envoy.api.v2.auth.CommonTlsContext\x12>\n\x1arequire_client_certificate\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12/\n\x0brequire_sni\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x46\n\x13session_ticket_keys\x18\x04 \x01(\x0b\x32\'.envoy.api.v2.auth.TlsSessionTicketKeysH\x00\x12S\n%session_ticket_keys_sds_secret_config\x18\x05 \x01(\x0b\x32\".envoy.api.v2.auth.SdsSecretConfigH\x00\x12.\n$disable_stateless_session_resumption\x18\x07 \x01(\x08H\x00\x12\x44\n\x0fsession_timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationB\x10\xfa\x42\r\xaa\x01\n\x1a\x06\x08\x80\x80\x80\x80\x10\x32\x00\x42\x1a\n\x18session_ticket_keys_type\"\x90\x06\n\x10\x43ommonTlsContext\x12\x34\n\ntls_params\x18\x01 \x01(\x0b\x32 .envoy.api.v2.auth.TlsParameters\x12;\n\x10tls_certificates\x18\x02 \x03(\x0b\x32!.envoy.api.v2.auth.TlsCertificate\x12X\n\"tls_certificate_sds_secret_configs\x18\x06 \x03(\x0b\x32\".envoy.api.v2.auth.SdsSecretConfigB\x08\xfa\x42\x05\x92\x01\x02\x10\x01\x12M\n\x12validation_context\x18\x03 \x01(\x0b\x32/.envoy.api.v2.auth.CertificateValidationContextH\x00\x12R\n$validation_context_sds_secret_config\x18\x07 \x01(\x0b\x32\".envoy.api.v2.auth.SdsSecretConfigH\x00\x12o\n\x1b\x63ombined_validation_context\x18\x08 \x01(\x0b\x32H.envoy.api.v2.auth.CommonTlsContext.CombinedCertificateValidationContextH\x00\x12\x16\n\x0e\x61lpn_protocols\x18\x04 \x03(\t\x1a\xe1\x01\n$CombinedCertificateValidationContext\x12]\n\x1a\x64\x65\x66\x61ult_validation_context\x18\x01 \x01(\x0b\x32/.envoy.api.v2.auth.CertificateValidationContextB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12Z\n$validation_context_sds_secret_config\x18\x02 \x01(\x0b\x32\".envoy.api.v2.auth.SdsSecretConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\x19\n\x17validation_context_typeJ\x04\x08\x05\x10\x06\x42\xa0\x01\n\x1fio.envoyproxy.envoy.api.v2.authB\x08TlsProtoP\x01Z8github.com/envoyproxy/go-control-plane/envoy/api/v2/auth\xf2\x98\xfe\x8f\x05+\x12)envoy.extensions.transport_sockets.tls.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.api.v2.auth.tls_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\037io.envoyproxy.envoy.api.v2.authB\010TlsProtoP\001Z8github.com/envoyproxy/go-control-plane/envoy/api/v2/auth\362\230\376\217\005+\022)envoy.extensions.transport_sockets.tls.v3\272\200\310\321\006\002\020\001'
  _globals['_UPSTREAMTLSCONTEXT'].fields_by_name['sni']._loaded_options = None
  _globals['_UPSTREAMTLSCONTEXT'].fields_by_name['sni']._serialized_options = b'\372B\005r\003(\377\001'
  _globals['_DOWNSTREAMTLSCONTEXT'].fields_by_name['session_timeout']._loaded_options = None
  _globals['_DOWNSTREAMTLSCONTEXT'].fields_by_name['session_timeout']._serialized_options = b'\372B\r\252\001\n\032\006\010\200\200\200\200\0202\000'
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT'].fields_by_name['default_validation_context']._loaded_options = None
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT'].fields_by_name['default_validation_context']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT'].fields_by_name['validation_context_sds_secret_config']._loaded_options = None
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT'].fields_by_name['validation_context_sds_secret_config']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_COMMONTLSCONTEXT'].fields_by_name['tls_certificate_sds_secret_configs']._loaded_options = None
  _globals['_COMMONTLSCONTEXT'].fields_by_name['tls_certificate_sds_secret_configs']._serialized_options = b'\372B\005\222\001\002\020\001'
  _globals['_UPSTREAMTLSCONTEXT']._serialized_start=267
  _globals['_UPSTREAMTLSCONTEXT']._serialized_end=460
  _globals['_DOWNSTREAMTLSCONTEXT']._serialized_start=463
  _globals['_DOWNSTREAMTLSCONTEXT']._serialized_end=966
  _globals['_COMMONTLSCONTEXT']._serialized_start=969
  _globals['_COMMONTLSCONTEXT']._serialized_end=1753
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT']._serialized_start=1495
  _globals['_COMMONTLSCONTEXT_COMBINEDCERTIFICATEVALIDATIONCONTEXT']._serialized_end=1720
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/ratelimit/v3/rate_limit.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/ratelimit/v3/rate_limit.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.ratelimit.v3 import rls_pb2 as envoy_dot_config_dot_ratelimit_dot_v3_dot_rls__pb2
from envoy.config.route.v3 import route_components_pb2 as envoy_dot_config_dot_route_dot_v3_dot_route__components__pb2
from envoy.type.v3 import http_status_pb2 as envoy_dot_type_dot_v3_dot_http__status__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;envoy/extensions/filters/http/ratelimit/v3/rate_limit.proto\x12*envoy.extensions.filters.http.ratelimit.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a#envoy/config/ratelimit/v3/rls.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\x1f\x65nvoy/type/v3/http_status.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xd5\x08\n\tRateLimit\x12\x17\n\x06\x64omain\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x16\n\x05stage\x18\x02 \x01(\rB\x07\xfa\x42\x04*\x02\x18\n\x12\x37\n\x0crequest_type\x18\x03 \x01(\tB!\xfa\x42\x1er\x1cR\x08internalR\x08\x65xternalR\x04\x62othR\x00\x12*\n\x07timeout\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x19\n\x11\x66\x61ilure_mode_deny\x18\x05 \x01(\x08\x12*\n\"rate_limited_as_resource_exhausted\x18\x06 \x01(\x08\x12W\n\x12rate_limit_service\x18\x07 \x01(\x0b\x32\x31.envoy.config.ratelimit.v3.RateLimitServiceConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x7f\n\x1a\x65nable_x_ratelimit_headers\x18\x08 \x01(\x0e\x32Q.envoy.extensions.filters.http.ratelimit.v3.RateLimit.XRateLimitHeadersRFCVersionB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12*\n\"disable_x_envoy_ratelimited_header\x18\t \x01(\x08\x12\x36\n\x13rate_limited_status\x18\n \x01(\x0b\x32\x19.envoy.type.v3.HttpStatus\x12R\n\x17response_headers_to_add\x18\x0b \x03(\x0b\x32\'.envoy.config.core.v3.HeaderValueOptionB\x08\xfa\x42\x05\x92\x01\x02\x10\n\x12\x32\n\x0fstatus_on_error\x18\x0c \x01(\x0b\x32\x19.envoy.type.v3.HttpStatus\x12\x13\n\x0bstat_prefix\x18\r \x01(\t\x12\x46\n\x0e\x66ilter_enabled\x18\x0e \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent\x12G\n\x0f\x66ilter_enforced\x18\x0f \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent\x12Q\n\x19\x66\x61ilure_mode_deny_percent\x18\x10 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent\x12\x35\n\x0brate_limits\x18\x11 \x03(\x0b\x32 .envoy.config.route.v3.RateLimit\"<\n\x1bXRateLimitHeadersRFCVersion\x12\x07\n\x03OFF\x10\x00\x12\x14\n\x10\x44RAFT_VERSION_03\x10\x01:7\x9a\xc5\x88\x1e\x32\n0envoy.config.filter.http.rate_limit.v2.RateLimit\"\xdb\x03\n\x11RateLimitPerRoute\x12s\n\x0evh_rate_limits\x18\x01 \x01(\x0e\x32Q.envoy.extensions.filters.http.ratelimit.v3.RateLimitPerRoute.VhRateLimitsOptionsB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12p\n\x0foverride_option\x18\x02 \x01(\x0e\x32M.envoy.extensions.filters.http.ratelimit.v3.RateLimitPerRoute.OverrideOptionsB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x35\n\x0brate_limits\x18\x03 \x03(\x0b\x32 .envoy.config.route.v3.RateLimit\x12\x0e\n\x06\x64omain\x18\x04 \x01(\t\"<\n\x13VhRateLimitsOptions\x12\x0c\n\x08OVERRIDE\x10\x00\x12\x0b\n\x07INCLUDE\x10\x01\x12\n\n\x06IGNORE\x10\x02\"Z\n\x0fOverrideOptions\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x13\n\x0fOVERRIDE_POLICY\x10\x01\x12\x12\n\x0eINCLUDE_POLICY\x10\x02\x12\x11\n\rIGNORE_POLICY\x10\x03\x42\xb3\x01\n8io.envoyproxy.envoy.extensions.filters.http.ratelimit.v3B\x0eRateLimitProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/ratelimit/v3;ratelimitv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.ratelimit.v3.rate_limit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n8io.envoyproxy.envoy.extensions.filters.http.ratelimit.v3B\016RateLimitProtoP\001Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/ratelimit/v3;ratelimitv3\272\200\310\321\006\002\020\002'
  _globals['_RATELIMIT'].fields_by_name['domain']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_RATELIMIT'].fields_by_name['stage']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['stage']._serialized_options = b'\372B\004*\002\030\n'
  _globals['_RATELIMIT'].fields_by_name['request_type']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['request_type']._serialized_options = b'\372B\036r\034R\010internalR\010externalR\004bothR\000'
  _globals['_RATELIMIT'].fields_by_name['rate_limit_service']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['rate_limit_service']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_RATELIMIT'].fields_by_name['enable_x_ratelimit_headers']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['enable_x_ratelimit_headers']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_RATELIMIT'].fields_by_name['response_headers_to_add']._loaded_options = None
  _globals['_RATELIMIT'].fields_by_name['response_headers_to_add']._serialized_options = b'\372B\005\222\001\002\020\n'
  _globals['_RATELIMIT']._loaded_options = None
  _globals['_RATELIMIT']._serialized_options = b'\232\305\210\0362\n0envoy.config.filter.http.rate_limit.v2.RateLimit'
  _globals['_RATELIMITPERROUTE'].fields_by_name['vh_rate_limits']._loaded_options = None
  _globals['_RATELIMITPERROUTE'].fields_by_name['vh_rate_limits']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_RATELIMITPERROUTE'].fields_by_name['override_option']._loaded_options = None
  _globals['_RATELIMITPERROUTE'].fields_by_name['override_option']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_RATELIMIT']._serialized_start=380
  _globals['_RATELIMIT']._serialized_end=1489
  _globals['_RATELIMIT_XRATELIMITHEADERSRFCVERSION']._serialized_start=1372
  _globals['_RATELIMIT_XRATELIMITHEADERSRFCVERSION']._serialized_end=1432
  _globals['_RATELIMITPERROUTE']._serialized_start=1492
  _globals['_RATELIMITPERROUTE']._serialized_end=1967
  _globals['_RATELIMITPERROUTE_VHRATELIMITSOPTIONS']._serialized_start=1815
  _globals['_RATELIMITPERROUTE_VHRATELIMITSOPTIONS']._serialized_end=1875
  _globals['_RATELIMITPERROUTE_OVERRIDEOPTIONS']._serialized_start=1877
  _globals['_RATELIMITPERROUTE_OVERRIDEOPTIONS']._serialized_end=1967
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides setuptools command classes for the GRPC Python setup process."""

# NOTE(https://github.com/grpc/grpc/issues/24028): allow setuptools to monkey
# patch distutils
import setuptools  # isort:skip

import glob
import os
import os.path
import shutil
import subprocess
import sys
import sysconfig
import tempfile
import traceback

from setuptools.command import build_ext
from setuptools.command import build_py
import support

PYTHON_STEM = os.path.dirname(os.path.abspath(__file__))
GRPC_STEM = os.path.abspath(PYTHON_STEM + "../../../../")
GRPC_ROOT = os.path.relpath(GRPC_STEM, start=GRPC_STEM)
PYTHON_REL_PATH = os.path.relpath(PYTHON_STEM, start=GRPC_STEM)
PROTO_STEM = os.path.join(GRPC_ROOT, "src", "proto")
PROTO_GEN_STEM = os.path.join(GRPC_ROOT, "src", "python", "gens")
CYTHON_STEM = os.path.join(PYTHON_REL_PATH, "grpc", "_cython")


class CommandError(Exception):
    """Simple exception class for GRPC custom commands."""


# TODO(atash): Remove this once PyPI has better Linux bdist support. See
# https://bitbucket.org/pypa/pypi/issues/120/binary-wheels-for-linux-are-not-supported
def _get_grpc_custom_bdist(decorated_basename, target_bdist_basename):
    """Returns a string path to a bdist file for Linux to install.

    If we can retrieve a pre-compiled bdist from online, uses it. Else, emits a
    warning and builds from source.
    """
    # TODO(atash): somehow the name that's returned from `wheel` is different
    # between different versions of 'wheel' (but from a compatibility standpoint,
    # the names are compatible); we should have some way of determining name
    # compatibility in the same way `wheel` does to avoid having to rename all of
    # the custom wheels that we build/upload to GCS.

    # Break import style to ensure that setup.py has had a chance to install the
    # relevant package.
    from urllib import request

    decorated_path = decorated_basename + GRPC_CUSTOM_BDIST_EXT
    try:
        url = BINARIES_REPOSITORY + "/{target}".format(target=decorated_path)
        bdist_data = request.urlopen(url).read()
    except IOError as error:
        raise CommandError(
            "{}\n\nCould not find the bdist {}: {}".format(
                traceback.format_exc(), decorated_path, error.message
            )
        )
    # Our chosen local bdist path.
    bdist_path = target_bdist_basename + GRPC_CUSTOM_BDIST_EXT
    try:
        with open(bdist_path, "w") as bdist_file:
            bdist_file.write(bdist_data)
    except IOError as error:
        raise CommandError(
            "{}\n\nCould not write grpcio bdist: {}".format(
                traceback.format_exc(), error.message
            )
        )
    return bdist_path


class SphinxDocumentation(setuptools.Command):
    """Command to generate documentation via sphinx."""

    description = "generate sphinx documentation"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # We import here to ensure that setup.py has had a chance to install the
        # relevant package eggs first.
        import sphinx.cmd.build

        source_dir = os.path.join(GRPC_ROOT, "doc", "python", "sphinx")
        target_dir = os.path.join(GRPC_ROOT, "doc", "build")
        exit_code = sphinx.cmd.build.build_main(
            ["-b", "html", "-W", "--keep-going", source_dir, target_dir]
        )
        if exit_code != 0:
            raise CommandError(
                "Documentation generation has warnings or errors"
            )


class BuildProjectMetadata(setuptools.Command):
    """Command to generate project metadata in a module."""

    description = "build grpcio project metadata files"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        module_file_path = os.path.join(
            PYTHON_REL_PATH, "grpc/_grpcio_metadata.py"
        )
        version = self.distribution.get_version()

        # TODO(sergiitk): sometime in Nov 2025 - consider removing the env var
        # and making this the default behavior.
        skip_metadata_update_on_match = os.environ.get(
            "GRPC_PYTHON_BUILD_SKIP_METADATA_ON_VERSION_MATCH", "0"
        )
        if skip_metadata_update_on_match == "1":
            with open(module_file_path, "r") as module_file:
                # prevent replacing the copyright header
                if self._get_version_in_grpcio_metadata(module_file) == version:
                    print(
                        f"Version match in _grpcio_metadata.py: {version},"
                        " skipping the update"
                    )
                    return

        with open(module_file_path, "w") as module_file:
            module_file.write(f'__version__ = """{version}"""')

    def _get_version_in_grpcio_metadata(self, module_file):
        try:
            import ast

            tree = ast.parse(module_file.read())
            for node in ast.walk(tree):
                if isinstance(node, ast.Assign):
                    for target in node.targets:
                        if isinstance(target, ast.Name):
                            variable_name = target.id
                            if variable_name != "__version__":
                                continue
                            if isinstance(node.value, ast.Constant):
                                return node.value.value
        except:
            raise

        return "-1"


class BuildPy(build_py.build_py):
    """Custom project build command."""

    def run(self):
        self.run_command("build_project_metadata")
        build_py.build_py.run(self)


def _poison_extensions(extensions, message):
    """Includes a file that will always fail to compile in all extensions."""

    # use relative path as setuptools doesn't support absolute path in
    # extension.sources
    poison_filename = os.path.join(PYTHON_REL_PATH, "poison.c")

    with open(poison_filename, "w") as poison:
        poison.write("#error {}".format(message))
    for extension in extensions:
        extension.sources = [poison_filename]


def check_and_update_cythonization(extensions):
    """Replace .pyx files with their generated counterparts and return whether or
    not cythonization still needs to occur."""
    for extension in extensions:
        generated_pyx_sources = []
        other_sources = []
        for source in extension.sources:
            base, file_ext = os.path.splitext(source)
            if file_ext == ".pyx":
                generated_pyx_source = next(
                    (
                        base + gen_ext
                        for gen_ext in (
                            ".c",
                            ".cpp",
                        )
                        if os.path.isfile(base + gen_ext)
                    ),
                    None,
                )
                if generated_pyx_source:
                    generated_pyx_sources.append(generated_pyx_source)
                else:
                    sys.stderr.write("Cython-generated files are missing...\n")
                    return False
            else:
                other_sources.append(source)
        extension.sources = generated_pyx_sources + other_sources
    sys.stderr.write("Found cython-generated files...\n")
    return True


def try_cythonize(extensions, linetracing=False, mandatory=True):
    """Attempt to cythonize the extensions.

    Args:
      extensions: A list of `setuptools.Extension`.
      linetracing: A bool indicating whether or not to enable linetracing.
      mandatory: Whether or not having Cython-generated files is mandatory. If it
        is, extensions will be poisoned when they can't be fully generated.
    """
    try:
        # Break import style to ensure we have access to Cython post-setup_requires
        import Cython.Build
    except ImportError:
        if mandatory:
            sys.stderr.write(
                "This package needs to generate C files with Cython but it"
                " cannot. Poisoning extension sources to disallow extension"
                " commands..."
            )
            _poison_extensions(
                extensions,
                (
                    "Extensions have been poisoned due to missing"
                    " Cython-generated code."
                ),
            )
        return extensions
    cython_compiler_directives = {}
    if linetracing:
        additional_define_macros = [("CYTHON_TRACE_NOGIL", "1")]
        cython_compiler_directives["linetrace"] = True
    return Cython.Build.cythonize(
        extensions,
        include_path=[
            include_dir
            for extension in extensions
            for include_dir in extension.include_dirs
        ]
        + [CYTHON_STEM],
        compiler_directives=cython_compiler_directives,
    )


class BuildExt(build_ext.build_ext):
    """Custom build_ext command to enable compiler-specific flags."""

    C_OPTIONS = {
        "unix": ("-pthread",),
        "msvc": (),
    }
    LINK_OPTIONS = {}

    def get_ext_filename(self, ext_name):
        # since python3.5, python extensions' shared libraries use a suffix that corresponds to the value
        # of sysconfig.get_config_var('EXT_SUFFIX') and contains info about the architecture the library targets.
        # E.g. on x64 linux the suffix is ".cpython-XYZ-x86_64-linux-gnu.so"
        # When crosscompiling python wheels, we need to be able to override this suffix
        # so that the resulting file name matches the target architecture and we end up with a well-formed
        # wheel.
        filename = build_ext.build_ext.get_ext_filename(self, ext_name)
        orig_ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
        new_ext_suffix = os.getenv("GRPC_PYTHON_OVERRIDE_EXT_SUFFIX")
        if new_ext_suffix and filename.endswith(orig_ext_suffix):
            filename = filename[: -len(orig_ext_suffix)] + new_ext_suffix
        return filename

    def build_extensions(self):

        # use short temp directory to avoid linker command file errors caused by
        # exceeding 131071 characters in Windows.
        # TODO(ssreenithi): Remove once we have a better solution: b/454497076
        use_short_temp = os.environ.get(
            "GRPC_PYTHON_BUILD_USE_SHORT_TEMP_DIR_NAME", 0
        )
        if use_short_temp == "1":
            if not os.path.exists("pyb"):
                os.mkdir("pyb")

            self.build_temp = tempfile.mkdtemp(dir="pyb")
            print(f"Using temp build directory: {self.build_temp}")

        # This is to let UnixCompiler get either C or C++ compiler options depending on the source.
        # Note that this doesn't work for MSVCCompiler and will be handled by _spawn_patch.py.
        old_compile = self.compiler._compile

        def new_compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
            # NOTE: keep in sync with setup.py EXTRA_ENV_COMPILE_ARGS.
            cpp_specific_args = {"-std=c++17", "-stdlib=libc++"}
            c_specific_args = {"-std=c11"}

            args_to_remove = set()
            if src.endswith(".c"):
                # Remove cpp-specific args when compiling c.
                args_to_remove = cpp_specific_args
            elif src.endswith((".cc", ".cpp")):
                # Remove c-specific args when compiling c++.
                args_to_remove = c_specific_args

            if args_to_remove:
                extra_postargs = [
                    arg for arg in extra_postargs if arg not in args_to_remove
                ]

            return old_compile(obj, src, ext, cc_args, extra_postargs, pp_opts)

        self.compiler._compile = new_compile

        compiler = self.compiler.compiler_type
        if compiler in BuildExt.C_OPTIONS:
            for extension in self.extensions:
                extension.extra_compile_args += list(
                    BuildExt.C_OPTIONS[compiler]
                )
        if compiler in BuildExt.LINK_OPTIONS:
            for extension in self.extensions:
                extension.extra_link_args += list(
                    BuildExt.LINK_OPTIONS[compiler]
                )
        if not check_and_update_cythonization(self.extensions):
            self.extensions = try_cythonize(self.extensions)
        try:
            build_ext.build_ext.build_extensions(self)
        except Exception as error:
            formatted_exception = traceback.format_exc()
            support.diagnose_build_ext_error(self, error, formatted_exception)
            raise CommandError(
                "Failed `build_ext` step:\n{}".format(formatted_exception)
            )


class Gather(setuptools.Command):
    """Command to gather project dependencies."""

    description = "gather dependencies for grpcio"
    user_options = [
        ("test", "t", "flag indicating to gather test dependencies"),
        ("install", "i", "flag indicating to gather install dependencies"),
    ]

    def initialize_options(self):
        self.test = False
        self.install = False

    def finalize_options(self):
        # distutils requires this override.
        pass

    def run(self):
        pass


class Clean(setuptools.Command):
    """Command to clean build artifacts."""

    description = "Clean build artifacts."
    user_options = [
        ("all", "a", "a phony flag to allow our script to continue"),
    ]

    _FILE_PATTERNS = (
        "pyb",
        "src/python/grpcio/__pycache__/",
        "src/python/grpcio/grpc/_cython/cygrpc.cpp",
        "src/python/grpcio/grpc/_cython/*.so",
        "src/python/grpcio/grpcio.egg-info/",
    )
    _CURRENT_DIRECTORY = os.path.normpath(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), "../../..")
    )

    def initialize_options(self):
        self.all = False

    def finalize_options(self):
        pass

    def run(self):
        for path_spec in self._FILE_PATTERNS:
            this_glob = os.path.normpath(
                os.path.join(Clean._CURRENT_DIRECTORY, path_spec)
            )
            abs_paths = glob.glob(this_glob)
            for path in abs_paths:
                if not str(path).startswith(Clean._CURRENT_DIRECTORY):
                    raise ValueError(
                        "Cowardly refusing to delete {}.".format(path)
                    )
                print("Removing {}".format(os.path.relpath(path)))
                if os.path.isfile(path):
                    os.remove(str(path))
                else:
                    shutil.rmtree(str(path))



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests server certificate rotation.

Here we test various aspects of gRPC Python, and in some cases gRPC
Core by extension, support for server certificate rotation.

* ServerSSLCertReloadTestWithClientAuth: test ability to rotate
  server's SSL cert for use in future channels with clients while not
  affecting any existing channel. The server requires client
  authentication.

* ServerSSLCertReloadTestWithoutClientAuth: like
  ServerSSLCertReloadTestWithClientAuth except that the server does
  not authenticate the client.

* ServerSSLCertReloadTestCertConfigReuse: tests gRPC Python's ability
  to deal with user's reuse of ServerCertificateConfiguration instances.
"""

import abc
import collections
from concurrent import futures
import logging
import os
import threading
import unittest

import grpc

from tests.testing import _application_common
from tests.testing import _server_application
from tests.testing.proto import services_pb2_grpc
from tests.unit import resources
from tests.unit import test_common

CA_1_PEM = resources.cert_hier_1_root_ca_cert()
CA_2_PEM = resources.cert_hier_2_root_ca_cert()

CLIENT_KEY_1_PEM = resources.cert_hier_1_client_1_key()
CLIENT_CERT_CHAIN_1_PEM = (
    resources.cert_hier_1_client_1_cert()
    + resources.cert_hier_1_intermediate_ca_cert()
)

CLIENT_KEY_2_PEM = resources.cert_hier_2_client_1_key()
CLIENT_CERT_CHAIN_2_PEM = (
    resources.cert_hier_2_client_1_cert()
    + resources.cert_hier_2_intermediate_ca_cert()
)

SERVER_KEY_1_PEM = resources.cert_hier_1_server_1_key()
SERVER_CERT_CHAIN_1_PEM = (
    resources.cert_hier_1_server_1_cert()
    + resources.cert_hier_1_intermediate_ca_cert()
)

SERVER_KEY_2_PEM = resources.cert_hier_2_server_1_key()
SERVER_CERT_CHAIN_2_PEM = (
    resources.cert_hier_2_server_1_cert()
    + resources.cert_hier_2_intermediate_ca_cert()
)

# for use with the CertConfigFetcher. Roughly a simple custom mock
# implementation
Call = collections.namedtuple("Call", ["did_raise", "returned_cert_config"])


def _create_channel(port, credentials):
    return grpc.secure_channel("localhost:{}".format(port), credentials)


def _create_client_stub(channel, expect_success):
    if expect_success:
        # per Nathaniel: there's some robustness issue if we start
        # using a channel without waiting for it to be actually ready
        grpc.channel_ready_future(channel).result(timeout=10)
    return services_pb2_grpc.FirstServiceStub(channel)


class CertConfigFetcher:
    def __init__(self):
        self._lock = threading.Lock()
        self._calls = []
        self._should_raise = False
        self._cert_config = None

    def reset(self):
        with self._lock:
            self._calls = []
            self._should_raise = False
            self._cert_config = None

    def configure(self, should_raise, cert_config):
        assert not (should_raise and cert_config), (
            "should not specify both should_raise and a cert_config at the same"
            " time"
        )
        with self._lock:
            self._should_raise = should_raise
            self._cert_config = cert_config

    def getCalls(self):
        with self._lock:
            return self._calls

    def __call__(self):
        with self._lock:
            if self._should_raise:
                self._calls.append(Call(True, None))
                raise ValueError("just for fun, should not affect the test")
            else:
                self._calls.append(Call(False, self._cert_config))
                return self._cert_config


class _ServerSSLCertReloadTest(unittest.TestCase, metaclass=abc.ABCMeta):
    def __init__(self, *args, **kwargs):
        super(_ServerSSLCertReloadTest, self).__init__(*args, **kwargs)
        self.server = None
        self.port = None

    @abc.abstractmethod
    def require_client_auth(self):
        raise NotImplementedError()

    def setUp(self):
        self.server = test_common.test_server()
        services_pb2_grpc.add_FirstServiceServicer_to_server(
            _server_application.FirstServiceServicer(), self.server
        )
        switch_cert_on_client_num = 10
        initial_cert_config = grpc.ssl_server_certificate_configuration(
            [(SERVER_KEY_1_PEM, SERVER_CERT_CHAIN_1_PEM)],
            root_certificates=CA_2_PEM,
        )
        self.cert_config_fetcher = CertConfigFetcher()
        server_credentials = grpc.dynamic_ssl_server_credentials(
            initial_cert_config,
            self.cert_config_fetcher,
            require_client_authentication=self.require_client_auth(),
        )
        self.port = self.server.add_secure_port("[::]:0", server_credentials)
        self.server.start()

    def tearDown(self):
        if self.server:
            self.server.stop(None)

    def _perform_rpc(self, client_stub, expect_success):
        # we don't care about the actual response of the rpc; only
        # whether we can perform it or not, and if not, the status
        # code must be UNAVAILABLE
        request = _application_common.UNARY_UNARY_REQUEST
        if expect_success:
            response = client_stub.UnUn(request)
            self.assertEqual(response, _application_common.UNARY_UNARY_RESPONSE)
        else:
            with self.assertRaises(grpc.RpcError) as exception_context:
                client_stub.UnUn(request)
            # If TLS 1.2 is used, then the client receives an alert message
            # before the handshake is complete, so the status is UNAVAILABLE. If
            # TLS 1.3 is used, then the client receives the alert message after
            # the handshake is complete, so the TSI handshaker returns the
            # TSI_PROTOCOL_FAILURE result. This result does not have a
            # corresponding status code, so this yields an UNKNOWN status.
            self.assertTrue(
                exception_context.exception.code()
                in [grpc.StatusCode.UNAVAILABLE, grpc.StatusCode.UNKNOWN]
            )

    def _do_one_shot_client_rpc(
        self,
        expect_success,
        root_certificates=None,
        private_key=None,
        certificate_chain=None,
    ):
        credentials = grpc.ssl_channel_credentials(
            root_certificates=root_certificates,
            private_key=private_key,
            certificate_chain=certificate_chain,
        )
        with _create_channel(self.port, credentials) as client_channel:
            client_stub = _create_client_stub(client_channel, expect_success)
            self._perform_rpc(client_stub, expect_success)

    def _test(self):
        # things should work...
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        # client should reject server...
        # fails because client trusts ca2 and so will reject server
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            False,
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertIsNone(call.returned_cert_config, "i= {}".format(i))

        # should work again...
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(True, None)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertTrue(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        # if with_client_auth, then client should be rejected by
        # server because client uses key/cert1, but server trusts ca2,
        # so server will reject
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            not self.require_client_auth(),
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_1_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_1_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertIsNone(call.returned_cert_config, "i= {}".format(i))

        # should work again...
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        # now create the "persistent" clients
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        channel_A = _create_channel(
            self.port,
            grpc.ssl_channel_credentials(
                root_certificates=CA_1_PEM,
                private_key=CLIENT_KEY_2_PEM,
                certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
            ),
        )
        persistent_client_stub_A = _create_client_stub(channel_A, True)
        self._perform_rpc(persistent_client_stub_A, True)
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        channel_B = _create_channel(
            self.port,
            grpc.ssl_channel_credentials(
                root_certificates=CA_1_PEM,
                private_key=CLIENT_KEY_2_PEM,
                certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
            ),
        )
        persistent_client_stub_B = _create_client_stub(channel_B, True)
        self._perform_rpc(persistent_client_stub_B, True)
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        # moment of truth!! client should reject server because the
        # server switch cert...
        cert_config = grpc.ssl_server_certificate_configuration(
            [(SERVER_KEY_2_PEM, SERVER_CERT_CHAIN_2_PEM)],
            root_certificates=CA_1_PEM,
        )
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, cert_config)
        self._do_one_shot_client_rpc(
            False,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertEqual(
                call.returned_cert_config, cert_config, "i= {}".format(i)
            )

        # now should work again...
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_1_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_1_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertIsNone(actual_calls[0].returned_cert_config)

        # client should be rejected by server if with_client_auth
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            not self.require_client_auth(),
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertIsNone(call.returned_cert_config, "i= {}".format(i))

        # here client should reject server...
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._do_one_shot_client_rpc(
            False,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertIsNone(call.returned_cert_config, "i= {}".format(i))

        # persistent clients should continue to work
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._perform_rpc(persistent_client_stub_A, True)
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 0)

        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, None)
        self._perform_rpc(persistent_client_stub_B, True)
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 0)

        channel_A.close()
        channel_B.close()


class ServerSSLCertConfigFetcherParamsChecks(unittest.TestCase):
    def test_check_on_initial_config(self):
        with self.assertRaises(TypeError):
            grpc.dynamic_ssl_server_credentials(None, str)
        with self.assertRaises(TypeError):
            grpc.dynamic_ssl_server_credentials(1, str)

    def test_check_on_config_fetcher(self):
        cert_config = grpc.ssl_server_certificate_configuration(
            [(SERVER_KEY_2_PEM, SERVER_CERT_CHAIN_2_PEM)],
            root_certificates=CA_1_PEM,
        )
        with self.assertRaises(TypeError):
            grpc.dynamic_ssl_server_credentials(cert_config, None)
        with self.assertRaises(TypeError):
            grpc.dynamic_ssl_server_credentials(cert_config, 1)


class ServerSSLCertReloadTestWithClientAuth(_ServerSSLCertReloadTest):
    def require_client_auth(self):
        return True

    test = _ServerSSLCertReloadTest._test


class ServerSSLCertReloadTestWithoutClientAuth(_ServerSSLCertReloadTest):
    def require_client_auth(self):
        return False

    test = _ServerSSLCertReloadTest._test


class ServerSSLCertReloadTestCertConfigReuse(_ServerSSLCertReloadTest):
    """Ensures that `ServerCertificateConfiguration` instances can be reused.

    Because gRPC Core takes ownership of the
    `grpc_ssl_server_certificate_config` encapsulated by
    `ServerCertificateConfiguration`, this test reuses the same
    `ServerCertificateConfiguration` instances multiple times to make sure
    gRPC Python takes care of maintaining the validity of
    `ServerCertificateConfiguration` instances, so that such instances can be
    re-used by user application.
    """

    def require_client_auth(self):
        return True

    def setUp(self):
        self.server = test_common.test_server()
        services_pb2_grpc.add_FirstServiceServicer_to_server(
            _server_application.FirstServiceServicer(), self.server
        )
        self.cert_config_A = grpc.ssl_server_certificate_configuration(
            [(SERVER_KEY_1_PEM, SERVER_CERT_CHAIN_1_PEM)],
            root_certificates=CA_2_PEM,
        )
        self.cert_config_B = grpc.ssl_server_certificate_configuration(
            [(SERVER_KEY_2_PEM, SERVER_CERT_CHAIN_2_PEM)],
            root_certificates=CA_1_PEM,
        )
        self.cert_config_fetcher = CertConfigFetcher()
        server_credentials = grpc.dynamic_ssl_server_credentials(
            self.cert_config_A,
            self.cert_config_fetcher,
            require_client_authentication=True,
        )
        self.port = self.server.add_secure_port("[::]:0", server_credentials)
        self.server.start()

    def test_cert_config_reuse(self):
        # succeed with A
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_A)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertEqual(
            actual_calls[0].returned_cert_config, self.cert_config_A
        )

        # fail with A
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_A)
        self._do_one_shot_client_rpc(
            False,
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_1_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_1_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertEqual(
                call.returned_cert_config, self.cert_config_A, "i= {}".format(i)
            )

        # succeed again with A
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_A)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertEqual(
            actual_calls[0].returned_cert_config, self.cert_config_A
        )

        # succeed with B
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_B)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_1_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_1_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertEqual(
            actual_calls[0].returned_cert_config, self.cert_config_B
        )

        # fail with B
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_B)
        self._do_one_shot_client_rpc(
            False,
            root_certificates=CA_1_PEM,
            private_key=CLIENT_KEY_2_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_2_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertGreaterEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        for i, call in enumerate(actual_calls):
            self.assertFalse(call.did_raise, "i= {}".format(i))
            self.assertEqual(
                call.returned_cert_config, self.cert_config_B, "i= {}".format(i)
            )

        # succeed again with B
        self.cert_config_fetcher.reset()
        self.cert_config_fetcher.configure(False, self.cert_config_B)
        self._do_one_shot_client_rpc(
            True,
            root_certificates=CA_2_PEM,
            private_key=CLIENT_KEY_1_PEM,
            certificate_chain=CLIENT_CERT_CHAIN_1_PEM,
        )
        actual_calls = self.cert_config_fetcher.getCalls()
        self.assertEqual(len(actual_calls), 1)
        self.assertFalse(actual_calls[0].did_raise)
        self.assertEqual(
            actual_calls[0].returned_cert_config, self.cert_config_B
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: xds/core/v3/resource.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'xds/core/v3/resource.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from xds.core.v3 import resource_name_pb2 as xds_dot_core_dot_v3_dot_resource__name__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1axds/core/v3/resource.proto\x12\x0bxds.core.v3\x1a\x19google/protobuf/any.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1fxds/core/v3/resource_name.proto\"l\n\x08Resource\x12\'\n\x04name\x18\x01 \x01(\x0b\x32\x19.xds.core.v3.ResourceName\x12\x0f\n\x07version\x18\x02 \x01(\t\x12&\n\x08resource\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyBU\n\x16\x63om.github.xds.core.v3B\rResourceProtoP\x01Z\"github.com/cncf/xds/go/xds/core/v3\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'xds.core.v3.resource_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.github.xds.core.v3B\rResourceProtoP\001Z\"github.com/cncf/xds/go/xds/core/v3\322\306\244\341\006\002\010\001'
  _globals['_RESOURCE']._serialized_start=136
  _globals['_RESOURCE']._serialized_end=244
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import http2_base_server
import messages_pb2

# Set the number of padding bytes per data frame to be very large
# relative to the number of data bytes for each data frame sent.
_LARGE_PADDING_LENGTH = 255
_SMALL_READ_CHUNK_SIZE = 5


class TestDataFramePadding:
    """
    In response to an incoming request, this test sends headers, followed by
    data, followed by a reset stream frame. Client asserts that the RPC failed.
    Client needs to deliver the complete message to the application layer.
    """

    def __init__(self, use_padding=True):
        self._base_server = http2_base_server.H2ProtocolBaseServer()
        self._base_server._handlers["DataReceived"] = self.on_data_received
        self._base_server._handlers["WindowUpdated"] = self.on_window_update
        self._base_server._handlers["RequestReceived"] = (
            self.on_request_received
        )

        # _total_updates maps stream ids to total flow control updates received
        self._total_updates = {}
        # zero window updates so far for connection window (stream id '0')
        self._total_updates[0] = 0
        self._read_chunk_size = _SMALL_READ_CHUNK_SIZE

        if use_padding:
            self._pad_length = _LARGE_PADDING_LENGTH
        else:
            self._pad_length = None

    def get_base_server(self):
        return self._base_server

    def on_data_received(self, event):
        logging.info(
            "on data received. Stream id: %d. Data length: %d"
            % (event.stream_id, len(event.data))
        )
        self._base_server.on_data_received_default(event)
        if len(event.data) == 0:
            return
        sr = self._base_server.parse_received_data(event.stream_id)
        stream_bytes = ""
        # Check if full grpc msg has been read into the recv buffer yet
        if sr:
            response_data = self._base_server.default_response_data(
                sr.response_size
            )
            logging.info(
                "Stream id: %d. total resp size: %d"
                % (event.stream_id, len(response_data))
            )
            # Begin sending the response. Add ``self._pad_length`` padding to each
            # data frame and split the whole message into data frames each carrying
            # only self._read_chunk_size of data.
            # The purpose is to have the majority of the data frame response bytes
            # be padding bytes, since ``self._pad_length`` >> ``self._read_chunk_size``.
            self._base_server.setup_send(
                response_data,
                event.stream_id,
                pad_length=self._pad_length,
                read_chunk_size=self._read_chunk_size,
            )

    def on_request_received(self, event):
        self._base_server.on_request_received_default(event)
        logging.info("on request received. Stream id: %s." % event.stream_id)
        self._total_updates[event.stream_id] = 0

    # Log debug info and try to resume sending on all currently active streams.
    def on_window_update(self, event):
        logging.info(
            "on window update. Stream id: %s. Delta: %s"
            % (event.stream_id, event.delta)
        )
        self._total_updates[event.stream_id] += event.delta
        total = self._total_updates[event.stream_id]
        logging.info(
            "... - total updates for stream %d : %d" % (event.stream_id, total)
        )
        self._base_server.on_window_update_default(
            event,
            pad_length=self._pad_length,
            read_chunk_size=self._read_chunk_size,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto\x12\x33\x65nvoy.extensions.filters.http.proto_api_scrubber.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1fxds/annotations/v3/status.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\"\xdf\x02\n\x16ProtoApiScrubberConfig\x12Z\n\x0e\x64\x65scriptor_set\x18\x01 \x01(\x0b\x32\x42.envoy.extensions.filters.http.proto_api_scrubber.v3.DescriptorSet\x12W\n\x0crestrictions\x18\x02 \x01(\x0b\x32\x41.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions\x12q\n\x0e\x66iltering_mode\x18\x03 \x01(\x0e\x32Y.envoy.extensions.filters.http.proto_api_scrubber.v3.ProtoApiScrubberConfig.FilteringMode\"\x1d\n\rFilteringMode\x12\x0c\n\x08OVERRIDE\x10\x00\"F\n\rDescriptorSet\x12\x35\n\x0b\x64\x61ta_source\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\"\x8c\x04\n\x0cRestrictions\x12v\n\x13method_restrictions\x18\x01 \x03(\x0b\x32Y.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions.MethodRestrictionsEntry\x12x\n\x14message_restrictions\x18\x02 \x03(\x0b\x32Z.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions.MessageRestrictionsEntry\x1a\x82\x01\n\x17MethodRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12V\n\x05value\x18\x02 \x01(\x0b\x32G.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions:\x02\x38\x01\x1a\x84\x01\n\x18MessageRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12W\n\x05value\x18\x02 \x01(\x0b\x32H.envoy.extensions.filters.http.proto_api_scrubber.v3.MessageRestrictions:\x02\x38\x01\"\xa7\x05\n\x12MethodRestrictions\x12\x89\x01\n\x1arequest_field_restrictions\x18\x01 \x03(\x0b\x32\x65.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions.RequestFieldRestrictionsEntry\x12\x8b\x01\n\x1bresponse_field_restrictions\x18\x02 \x03(\x0b\x32\x66.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions.ResponseFieldRestrictionsEntry\x12\x62\n\x12method_restriction\x18\x03 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig\x1a\x87\x01\n\x1dRequestFieldRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12U\n\x05value\x18\x02 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig:\x02\x38\x01\x1a\x88\x01\n\x1eResponseFieldRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12U\n\x05value\x18\x02 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig:\x02\x38\x01\"m\n\x13MessageRestrictions\x12V\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig\"B\n\x11RestrictionConfig\x12-\n\x07matcher\x18\x01 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\xd3\x01\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\x0b\x43onfigProtoP\x01Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.proto_api_scrubber.v3.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\013ConfigProtoP\001Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._loaded_options = None
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._loaded_options = None
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._loaded_options = None
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._loaded_options = None
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_PROTOAPISCRUBBERCONFIG']._serialized_start=254
  _globals['_PROTOAPISCRUBBERCONFIG']._serialized_end=605
  _globals['_PROTOAPISCRUBBERCONFIG_FILTERINGMODE']._serialized_start=576
  _globals['_PROTOAPISCRUBBERCONFIG_FILTERINGMODE']._serialized_end=605
  _globals['_DESCRIPTORSET']._serialized_start=607
  _globals['_DESCRIPTORSET']._serialized_end=677
  _globals['_RESTRICTIONS']._serialized_start=680
  _globals['_RESTRICTIONS']._serialized_end=1204
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_start=939
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_end=1069
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_start=1072
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_end=1204
  _globals['_METHODRESTRICTIONS']._serialized_start=1207
  _globals['_METHODRESTRICTIONS']._serialized_end=1886
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_start=1612
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_end=1747
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_start=1750
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_end=1886
  _globals['_MESSAGERESTRICTIONS']._serialized_start=1888
  _globals['_MESSAGERESTRICTIONS']._serialized_end=1997
  _globals['_RESTRICTIONCONFIG']._serialized_start=1999
  _globals['_RESTRICTIONCONFIG']._serialized_end=2065
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of RPCs made against gRPC Python's application-layer API."""

import logging
import unittest

import grpc

from tests.unit.framework.common import test_constants

_SERIALIZE_REQUEST = lambda bytestring: bytestring * 2
_DESERIALIZE_REQUEST = lambda bytestring: bytestring[len(bytestring) // 2 :]
_SERIALIZE_RESPONSE = lambda bytestring: bytestring * 3
_DESERIALIZE_RESPONSE = lambda bytestring: bytestring[: len(bytestring) // 3]

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"


def _unary_unary_multi_callable(channel):
    return channel.unary_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
        _registered_method=True,
    )


def _unary_stream_multi_callable(channel):
    return channel.unary_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
        request_serializer=_SERIALIZE_REQUEST,
        response_deserializer=_DESERIALIZE_RESPONSE,
        _registered_method=True,
    )


def _stream_unary_multi_callable(channel):
    return channel.stream_unary(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
        request_serializer=_SERIALIZE_REQUEST,
        response_deserializer=_DESERIALIZE_RESPONSE,
        _registered_method=True,
    )


def _stream_stream_multi_callable(channel):
    return channel.stream_stream(
        grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
        _registered_method=True,
    )


class InvalidMetadataTest(unittest.TestCase):
    def setUp(self):
        self._channel = grpc.insecure_channel("localhost:8080")
        self._unary_unary = _unary_unary_multi_callable(self._channel)
        self._unary_stream = _unary_stream_multi_callable(self._channel)
        self._stream_unary = _stream_unary_multi_callable(self._channel)
        self._stream_stream = _stream_stream_multi_callable(self._channel)

    def tearDown(self):
        self._channel.close()

    def testUnaryRequestBlockingUnaryResponse(self):
        request = b"\x07\x08"
        metadata = (("InVaLiD", "UnaryRequestBlockingUnaryResponse"),)
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._unary_unary(request, metadata=metadata)

        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testUnaryRequestBlockingUnaryResponseWithCall(self):
        request = b"\x07\x08"
        metadata = (("InVaLiD", "UnaryRequestBlockingUnaryResponseWithCall"),)
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._unary_unary.with_call(request, metadata=metadata)

        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testUnaryRequestFutureUnaryResponse(self):
        request = b"\x07\x08"
        metadata = (("InVaLiD", "UnaryRequestFutureUnaryResponse"),)
        response_future = self._unary_unary.future(request, metadata=metadata)

        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testUnaryRequestStreamResponse(self):
        request = b"\x37\x58"
        metadata = (("InVaLiD", "UnaryRequestStreamResponse"),)
        response_iterator = self._unary_stream(request, metadata=metadata)

        with self.assertRaises(grpc.RpcError) as exception_context:
            list(response_iterator)
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testStreamRequestBlockingUnaryResponse(self):
        request_iterator = (
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        metadata = (("InVaLiD", "StreamRequestBlockingUnaryResponse"),)
        with self.assertRaises(grpc.RpcError) as exception_context:
            self._stream_unary(request_iterator, metadata=metadata)

        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testStreamRequestBlockingUnaryResponseWithCall(self):
        request_iterator = (
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        metadata = (("InVaLiD", "StreamRequestBlockingUnaryResponseWithCall"),)
        multi_callable = _stream_unary_multi_callable(self._channel)
        with self.assertRaises(grpc.RpcError) as exception_context:
            multi_callable.with_call(request_iterator, metadata=metadata)

        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testStreamRequestFutureUnaryResponse(self):
        request_iterator = (
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        metadata = (("InVaLiD", "StreamRequestFutureUnaryResponse"),)
        response_future = self._stream_unary.future(
            request_iterator, metadata=metadata
        )

        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testStreamRequestStreamResponse(self):
        request_iterator = (
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        metadata = (("InVaLiD", "StreamRequestStreamResponse"),)
        response_iterator = self._stream_stream(
            request_iterator, metadata=metadata
        )

        with self.assertRaises(grpc.RpcError) as exception_context:
            list(response_iterator)
        self.assertEqual(
            grpc.StatusCode.INTERNAL, exception_context.exception.code()
        )

    def testInvalidMetadata(self):
        self.assertRaises(TypeError, self._unary_unary, b"", metadata=42)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Defines a number of module-scope gRPC scenarios to test server shutdown."""

import argparse
from concurrent import futures
import logging
import os
import queue
import threading
import time

import grpc

from tests.unit import test_common

WAIT_TIME = 1000

REQUEST = b"request"
RESPONSE = b"response"

SERVER_RAISES_EXCEPTION = "server_raises_exception"
SERVER_DEALLOCATED = "server_deallocated"
SERVER_FORK_CAN_EXIT = "server_fork_can_exit"

_SERVICE_NAME = "test"
FORK_EXIT = "ForkExit"


def fork_and_exit(request, servicer_context):
    pid = os.fork()
    if pid == 0:
        os._exit(0)
    return RESPONSE


class GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == FORK_EXIT:
            return grpc.unary_unary_rpc_method_handler(fork_and_exit)
        else:
            return None


_METHOD_HANDLERS = {
    FORK_EXIT: grpc.unary_unary_rpc_method_handler(fork_and_exit)
}


def run_server(port_queue):
    server = test_common.test_server()
    port = server.add_insecure_port("[::]:0")
    port_queue.put(port)
    server.add_registered_method_handlers(_SERVICE_NAME, _METHOD_HANDLERS)
    server.start()
    # threading.Event.wait() does not exhibit the bug identified in
    # https://github.com/grpc/grpc/issues/17093, sleep instead
    time.sleep(WAIT_TIME)


def run_test(args):
    if args.scenario == SERVER_RAISES_EXCEPTION:
        server = test_common.test_server()
        server.start()
        raise Exception()
    elif args.scenario == SERVER_DEALLOCATED:
        server = test_common.test_server()
        server.start()
        server.__del__()
        while server._state.stage != grpc._server._ServerStage.STOPPED:
            pass
    elif args.scenario == SERVER_FORK_CAN_EXIT:
        port_queue = queue.Queue()
        thread = threading.Thread(target=run_server, args=(port_queue,))
        thread.daemon = True
        thread.start()
        port = port_queue.get()
        channel = grpc.insecure_channel("localhost:%d" % port)
        multi_callable = channel.unary_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, FORK_EXIT),
            _registered_method=True,
        )
        result, call = multi_callable.with_call(REQUEST, wait_for_ready=True)
        os.wait()
    else:
        raise ValueError("unknown test scenario")


if __name__ == "__main__":
    logging.basicConfig()
    parser = argparse.ArgumentParser()
    parser.add_argument("scenario", type=str)
    args = parser.parse_args()
    run_test(args)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import site
import sys

_GRPC_BAZEL_RUNTIME_ENV = "GRPC_BAZEL_RUNTIME"


# TODO(https://github.com/bazelbuild/bazel/issues/6844) Bazel failed to
# interpret namespace packages correctly. This monkey patch will force the
# Python process to parse the .pth file in the sys.path to resolve namespace
# package in the right place.
# Analysis in depth: https://github.com/bazelbuild/rules_python/issues/55
def sys_path_to_site_dir_hack():
    """Add valid sys.path item to site directory to parse the .pth files."""
    # Only run within our Bazel environment
    if not os.environ.get(_GRPC_BAZEL_RUNTIME_ENV):
        return
    items = []
    for item in sys.path:
        if os.path.exists(item):
            # The only difference between sys.path and site-directory is
            # whether the .pth file will be parsed or not. A site-directory
            # will always exist in sys.path, but not another way around.
            items.append(item)
    for item in items:
        site.addsitedir(item)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of credentials."""

import logging
import unittest

import grpc


class CredentialsTest(unittest.TestCase):
    def test_call_credentials_composition(self):
        first = grpc.access_token_call_credentials("abc")
        second = grpc.access_token_call_credentials("def")
        third = grpc.access_token_call_credentials("ghi")

        first_and_second = grpc.composite_call_credentials(first, second)
        first_second_and_third = grpc.composite_call_credentials(
            first, second, third
        )

        self.assertIsInstance(first_and_second, grpc.CallCredentials)
        self.assertIsInstance(first_second_and_third, grpc.CallCredentials)

    def test_channel_credentials_composition(self):
        first_call_credentials = grpc.access_token_call_credentials("abc")
        second_call_credentials = grpc.access_token_call_credentials("def")
        third_call_credentials = grpc.access_token_call_credentials("ghi")
        channel_credentials = grpc.ssl_channel_credentials()

        channel_and_first = grpc.composite_channel_credentials(
            channel_credentials, first_call_credentials
        )
        channel_first_and_second = grpc.composite_channel_credentials(
            channel_credentials, first_call_credentials, second_call_credentials
        )
        channel_first_second_and_third = grpc.composite_channel_credentials(
            channel_credentials,
            first_call_credentials,
            second_call_credentials,
            third_call_credentials,
        )

        self.assertIsInstance(channel_and_first, grpc.ChannelCredentials)
        self.assertIsInstance(channel_first_and_second, grpc.ChannelCredentials)
        self.assertIsInstance(
            channel_first_second_and_third, grpc.ChannelCredentials
        )

    def test_invalid_string_certificate(self):
        self.assertRaises(
            TypeError,
            grpc.ssl_channel_credentials,
            root_certificates="A Certificate",
            private_key=None,
            certificate_chain=None,
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)
